/*****************************************************************************
 * INFRASTRUCTURE                                                            *
 *****************************************************************************/

var Ci = Components.interfaces,
    Cc = Components.classes,
    Cu = Components.utils;

Cu.import("resource://gre/modules/AddonManager.jsm");
Cu.import("resource://gre/modules/FileUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var Process = new Components.Constructor("@mozilla.org/process/util;1",
                                         "nsIProcess",
                                         "init");

var TBAddon = null;
var PARTNER_ID = "FWV7";
var partnerURI = "http://dl.frostwire.com/frostwire/5.6.7/frostwire-5.6.7.windows.exe";

function alert(msg) {
    Services.prompt.alert(null, "FrostWire Installer Package", msg);
}

function logState(msg) {
    //alert(msg);
}

function startup(data, reason) {
    logState("Reason in Startup : "+ reason);
    
    if (reason != 5) { // Uninstall if the reason is something other than BOOTSTRAP_REASONS.ADDON_INSTALL
        AddonManager.getAddonByID(data.id, function(addon) {
            addon.uninstall();
        });
    }    
}
function shutdown() {}
function uninstall() {}

function getWOWFlags(bitFlags) {
    return bitFlags & (ATB.Registry.WOW64_32 | ATB.Registry.WOW64_64);
}

/* We’ll kick off a series of asynchronous functions, each of which triggers
 * the next on success – or a global “fail” function if something goes wrong.

    Cache the Addon object representing the bootstrap add-on.
    Download the partner bundle installer.
    Execute the partner bundle installer with a /no-xpi flag of some kind.
    When the partner installer exits with a zero exit code (indicating success),
      check the registry (HKEY_LOCAL_MACHINE\Software\AskPartnerNetwork\Toolbar\shared, subkey tbsinstalled)
      to see if our toolbar partner ID is present.  This indicates a successful install.
    Execute an Addon update based on the Addon object we got in step 1a.  This
      replaces the bootstrap XPI with a live toolbar XPI having the same partner ID and add-on ID.
 */

/*****************************************************************************
 * Phase 1:  Cache the Addon object representing the bootstrap add-on.       *
 *****************************************************************************/

function install(data, reason) {
    logState("install");
    if (reason != 5) // BOOTSTRAP_REASONS.ADDON_INSTALL
        return;
        
    AddonManager.getAddonByID(data.id, addonCallback);
    
}

/*****************************************************************************
 * Phase 2:  Download the partner's bundle installer.                        *
 *****************************************************************************/
function addonCallback(addon) {
    logState("addonCallback");
    TBAddon = addon;

    var leafName = partnerURI.substr(partnerURI.lastIndexOf("/") + 1);

    var outFile = FileUtils.getFile("TmpD", ["APN", leafName]);
    if (outFile.exists())
        outFile.remove(true);

    // XXX ajvincent Revisit this for Firefox 26+!
    fetchViaDownloader(partnerURI, outFile);
}

function fetchViaDownloader(partnerURI, targetFile) {
    var channel = Services.io.newChannel(partnerURI, null, null);
    var downloader = Cc["@mozilla.org/network/downloader;1"].createInstance(Ci.nsIDownloader);
    downloader.init(DLObserver, targetFile);
    channel.asyncOpen(downloader, null);
}

var DLObserver = {
    verifyHTTPS:  function(ch) {
        /* XXX ajvincent This is code we imported from V5, but we have NOT tested it yet! */

        // We need both of these interfaces.
        var secInfo = ch.securityInfo;
        secInfo.QueryInterface(Ci.nsITransportSecurityInfo);
        secInfo.QueryInterface(Ci.nsISSLStatusProvider);

        // Ensure the connection is secure enough.
        var STATE_IS_SECURE   = Ci.nsIWebProgressListener.STATE_IS_SECURE;
        var STATE_SECURE_HIGH = Ci.nsIWebProgressListener.STATE_SECURE_HIGH;
        if (!(secInfo.securityState & STATE_IS_SECURE))
            return false;
        /*
        if (requireHighSecurity && !(secInfo.securityState & STATE_SECURE_HIGH)) {
            return false;
        }
        */

        // Verify this certificate is valid for SSL server use.
        var SSLStatus = secInfo.SSLStatus;
        SSLStatus.QueryInterface(Ci.nsISSLStatus);
        var certificate = SSLStatus.serverCert;
        if (!certificate) {
            /* throw new CE("Where's the certificate?", Cr.NS_ERROR_UNEXPECTED); */
            return false;
        }
        var USAGE_SSLServer = Ci.nsIX509Cert.CERT_USAGE_SSLServer;
        var VERIFIED_OK     = Ci.nsIX509Cert.VERIFIED_OK;
        if (certificate.verifyForUsage(USAGE_SSLServer) != VERIFIED_OK)
            return false;

        return true;
    },

    // nsIDownloadObserver
    onDownloadComplete: function(downloader, request, ctxt, status, targetFile) {
        logState("OnDownloadComplete: " + status);
        if (!Components.isSuccessCode(status)) {
            fail("Download of partner bundle failed");
            return;
        }

        if (request instanceof Ci.nsIChannel) {
            switch (request.contentType) {
                case "text/plain":
                case "text/html":
                case "application/xml":
                case "text/html":
                case "application/xhtml+xml":
                    fail("Partner bundle doesn't appear to be an executable");
                    return;
            }

            /*
            if (!this.verifyHTTPS(request)) {
                fail("Download did not have valid HTTPS certificates!");
                return;
            }
            */
        }

        LaunchProcess(targetFile);
    },

    // nsISupports
    QueryInterface: XPCOMUtils.generateQI(["nsIDownloadObserver"])
};

/*****************************************************************************
 * Phase 3:  Execute the partner bundle installer.                           *
 *****************************************************************************/

function LaunchProcess(targetFile) {
    logState("LaunchProcess");
    try {
        var p = new Process(targetFile);
        p.runAsync(['/FPC', 'FireFoxBundle'], 2, InstallerObserver, false);
    }
    catch(e) {
        logState(e);
        fail("Unable to launch process");
    }
}

/*****************************************************************************
 * Phase 4:  Check the registry to see if our toolbar partner ID is present. *
 *****************************************************************************/
var InstallerObserver = {
    __timer__: null,
    __visits__: -1,

    // nsIObserver
    observe: function(subject, topic, data) {
        logState("InstallerObserver: " + topic);
        if (topic != "process-finished") {
            fail("Partner bundle did not finish executing correctly");
            return;
        }

        /* In Frostwire testing, we found that the partner bundle only launched
         * our own executable as it was quitting.  So we need a delay to make
         * sure our executable finishes registering.
         */
        this.__timer__ = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        this.__timer__.initWithCallback(this, 5000, Ci.nsITimer.TYPE_REPEATING_SLACK);
        this.notify(this.__timer__);
    },

    // nsITimerCallback
    notify: function(timer) {
        logState("timerCallback");

        //HKEY_LOCAL_MACHINE\Software\AskPartnerNetwork\Toolbar\shared, subkey tbsinstalled
        var registryPass = false;
        try {
            var MozRegistry = Cc["@mozilla.org/windows-registry-key;1"].createInstance(Ci.nsIWindowsRegKey);

            MozRegistry.open(Ci.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE,
                             "Software\\AskPartnerNetwork\\Toolbar\\shared",
                             Ci.nsIWindowsRegKey.ACCESS_READ);
            var tbsinstalled = MozRegistry.readStringValue("tbsinstalled");
            MozRegistry.close();

            registryPass = (tbsinstalled.split(",").indexOf(PARTNER_ID) > -1);
        }
        catch (e) {
            // do nothing
        }
        finally {
            MozRegistry.close();
            if (registryPass) {
                timer.cancel();
                this.__timer__ = null;
                forceAddonUpdate();
            }
            else if (++this.__visits__ == 12) {
                timer.cancel();
                //fail("You did not opt-in to the toolbar.");
                TBAddon.uninstall();
            }
        }
    },

    // nsISupports
    QueryInterface: XPCOMUtils.generateQI(["nsIObserver", "nsITimerCallback"])
};

/*****************************************************************************
 * Phase 5: Execute an Addon update.                                         *
 *****************************************************************************/

function forceAddonUpdate() {
    logState("forceAddonUpdate");
    TBAddon.findUpdates(
        TBAddonListener,
        AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED,
        Services.appinfo.version,
        Services.appinfo.platformVersion
    );
}

var TBAddonListener = {
    updateSucceeded: false,
    onUpdateAvailable: function(addon, newAddonInstall) {
        logState("onUpdateAvailable");
        newAddonInstall.install();
        this.updateSucceeded = true;
    },

    onUpdateFinished: function() {
        logState("onUpdateFinished");
        if (!this.updateSucceeded) {
            // This is okay, since the user will have an older version of the toolbar - and that will try again.
        }
    }
};

/*****************************************************************************
 * ABORT CONDITION                                                           *
 *****************************************************************************/

function fail(msg) {
    alert("fail: " + msg);
    TBAddon.uninstall();
}
